Attribute VB_Name = "modWic"
' Autor: F. Schler
' Datum: 08/2020

Option Explicit

' ----==== Const ====----
Public Const S_OK As Long = &H0&
Private Const CLSCTX_INPROC As Long = &H1&
Private Const CC_STDCALL As Long = &H4&
Private Const IID_Release As Long = &H8&

Public Const CLSID_WICImagingFactory As String = "{cacaf262-9370-4615-a13b-9f5539da4c0a}"
Public Const CLSID_WICImagingFactory1 As String = CLSID_WICImagingFactory
Public Const CLSID_WICImagingFactory2 As String = "{317d06e8-5f24-433d-bdf7-79ce68d8abc2}"
Public Const IID_IWICImagingFactory As String = "{ec5ec8a9-c395-4314-9c77-54d7a935ff70}"
Public Const IID_IWICBitmapCodecInfo As String = "{e87a44c4-b76e-4c47-8b09-298eb12a2714}"

' ----==== Enums ====----
Public Enum vtb_Interfaces
    
    ' IUnknown
    QueryInterface = 0
    
    ' IEnumUnknown
    Fetch = 3 '(Next)
    
    ' IWICComponentInfo
    GetComponentType = 3                'WICComponentType
    GetCLSID = 4                        'CLSID
    GetSigningStatus = 5                'WICComponentSigning
    GetAuthor = 6                       'String
    GetVendorGUID = 7                   'GUID
    GetVersion = 8                      'String
    GetSpecVersion = 9                  'String
    
    ' IWICBitmapCodecInfo
    GetFriendlyName = 10                'String
    GetContainerFormat = 11             'GUID
    GetPixelFormats = 12                'Array GUID
    GetColorManagementVersion = 13      'String
    GetDeviceManufacturer = 14          'String
    GetDeviceModels = 15                'String
    GetMimeTypes = 16                   'String
    GetFileExtensions = 17              'String
    DoesSupportAnimation = 18           'Boolean
    DoesSupportChromaKey = 19           'Boolean
    DoesSupportLossless = 20            'Boolean
    DoesSupportMultiframe = 21          'Boolean
    MatchesMimeType = 22                'Boolean
    
    ' IWICFactory
    CreateComponentEnumerator = 23
    
End Enum

Public Enum WICComponentSigning
    WICComponentSigned = &H1&
    WICComponentUnsigned = &H2&
    WICComponentSafe = &H4&
    WICComponentDisabled = &H80000000
End Enum

Public Enum WICComponentType
    WICDecoder = &H1&
    WICEncoder = &H2&
    WICPixelFormatConverter = &H4&
    WICMetadataReader = &H8&
    WICMetadataWriter = &H10&
    WICPixelFormat = &H20&
End Enum

Public Enum WICComponentEnumerateOptions
    WICComponentEnumerateDefault = &H0&
    WICComponentEnumerateRefresh = &H1&
    WICComponentEnumerateBuiltInOnly = &H20000000
    WICComponentEnumerateUnsigned = &H40000000
    WICComponentEnumerateDisabled = &H80000000
End Enum

' ----==== Type ====----
Public Type GUID
    data1 As Long
    data2 As Integer
    data3 As Integer
    data4(0 To 7) As Byte
End Type

Public Type CLSID
    data1 As Long
    data2 As Integer
    data3 As Integer
    data4(0 To 7) As Byte
End Type

' ----==== Kernel32.dll API-Deklarationen ====----
Private Declare Sub CopyMemory Lib "Kernel32.dll" _
                    Alias "RtlMoveMemory" ( _
                    ByRef Destination As Any, _
                    ByRef Source As Any, _
                    ByVal length As Long)

Private Declare Function lstrlenW Lib "Kernel32.dll" ( _
                         ByVal lpString As Long) As Long

' ----==== OleAut32 API-Deklarationen ====----
Private Declare Function DispCallFunc Lib "OleAut32" ( _
                    ByVal pvInstance As Long, _
                    ByVal oVft As Long, _
                    ByVal cc As Long, _
                    ByVal vtReturn As VbVarType, _
                    ByVal cActuals As Long, _
                    ByVal prgvt As Any, _
                    ByVal prgpvarg As Any, _
                    ByRef pvargResult As Variant) As Long

' ----==== Ole32 API-Deklarationen ====----
Private Declare Sub CoTaskMemFree Lib "Ole32.dll" ( _
                    ByVal hMem As Long)

Private Declare Function CLSIDFromString Lib "Ole32.dll" ( _
                         ByVal pstring As Long, _
                         ByRef pclsid As GUID) As Long

Private Declare Function StringFromGUID Lib "Ole32.dll" _
                         Alias "StringFromCLSID" ( _
                         ByRef pclsid As GUID, _
                         ByRef lpszProgID As Long) As Long

Private Declare Function StringFromCLSID Lib "Ole32.dll" ( _
                         ByRef pclsid As CLSID, _
                         ByRef lpszProgID As Long) As Long

Private Declare Function CoCreateInstance Lib "ole32" ( _
                         ByVal rclsid As Long, _
                         ByVal pUnkOuter As Long, _
                         ByVal dwClsContext As Long, _
                         ByVal riid As Long, _
                         ByRef ppv As Long) As Long

' ----==== Create Interface ====----
Public Function CreateInterface(ByVal sCLSID As String, _
    ByVal sIID As String, ByRef pInterface As Long) As Boolean

    ' Interface erstellen
    If CoCreateInstance(VarPtr(Str2Guid(sCLSID)), 0&, CLSCTX_INPROC, _
        VarPtr(Str2Guid(sIID)), pInterface) = S_OK Then

        CreateInterface = True

    End If

End Function

' ----==== Release Interface ====----
Public Sub ReleaseInterface(ByRef pInterface As Long)

    Dim vRet As Variant

    ' ist ein Pointer auf ein Interfac vorhanden
    If pInterface <> 0& Then

        ' Funktion Release des Interfaces aufrufen
        If DispCallFunc(pInterface, IID_Release, CC_STDCALL, vbLong, _
            0&, 0&, 0&, vRet) = S_OK Then

            pInterface = 0&

        End If

    End If

End Sub

' ----==== Call Interface Function ====----
Public Function Invoke(ByVal pInterface As Long, ByVal vtb As vtb_Interfaces, _
    ParamArray var()) As Variant
    
    If pInterface <> 0& Then
    
        Invoke = OleInvoke(pInterface, vtb, var)
        
    End If
    
End Function

Private Function OleInvoke(ByVal pInterface As Long, ByVal lCmd As Long, _
    ParamArray aParam()) As Variant
    
    Dim lCount As Long
    Dim lItem As Long
    Dim vRet As Variant
    Dim vParam As Variant
    Dim olePtr(10) As Long
    Dim oleTyp(10) As Integer

    ' ist ein Pointer auf ein Interface vorhanden
    If pInterface <> 0& Then

        ' nur wenn zum Aufruf der Interface-Funktion auch
        ' Parameter vorhanden sind.
        If UBound(aParam) >= 0 Then

            ' ParamArray nach Variant
            vParam = aParam

            ' ist der Variant ein Array
            If IsArray(vParam) Then vParam = vParam(0)

            ' Anzahl der Parameter
            lCount = UBound(vParam)

            ' alle Parameter durchlaufen
            For lItem = 0 To lCount

                ' Typ des Parameter
                oleTyp(lItem) = VarType(vParam(lItem))

                ' Pointer auf den Parameter
                olePtr(lItem) = VarPtr(vParam(lItem))

            Next

        End If

        ' Funktion des Interfaces aufrufen
        If DispCallFunc(pInterface, lCmd * 4, CC_STDCALL, vbLong, _
            lItem, VarPtr(oleTyp(0)), VarPtr(olePtr(0)), vRet) <> S_OK Then
            
            Debug.Print "Fehler beim Aufrufen der Interface-Funktion!"

        End If

    End If

    OleInvoke = vRet

End Function

' ----==== Helper Function ====----
Public Function Str2Guid(ByVal sStr As String) As GUID

    If Len(sStr) > 0& Then

        Call CLSIDFromString(StrPtr(sStr), Str2Guid)
    
    End If
    
End Function

Public Function Guid2Str(ByRef tGUID As GUID) As String

    Dim lGuid As Long

    Call StringFromGUID(tGUID, lGuid)

    Guid2Str = Ptr2Str(lGuid)

End Function

Public Function Clsid2Str(ByRef tCLSID As CLSID) As String

    Dim lClsid As Long

    Call StringFromCLSID(tCLSID, lClsid)

    Clsid2Str = Ptr2Str(lClsid)

End Function

Private Function Ptr2Str(ByVal lpStrPointer As Long) As String

    Dim lLen As Long
    Dim bBuffer() As Byte

    lLen = (lstrlenW(lpStrPointer) * 2) - 1

    If lLen > 0 Then

        ReDim bBuffer(lLen)

        Call CopyMemory(bBuffer(0), ByVal lpStrPointer, lLen)
        
        Call CoTaskMemFree(lpStrPointer)

        Ptr2Str = bBuffer

    End If

End Function

' ----==== Helper Function nur fr IWICComponentInfo und IWICBitmapCodecInfo ====----
Public Function GetCodecInfoString(ByVal pInterface As Long, _
    ByVal vtb As vtb_Interfaces) As String

    Dim wzString As String
    Dim cchString As Long
    Dim cchActual As Long

    ' ist ein Pointer auf ein Interface vorhanden
    If pInterface <> 0& Then

        ' erster Aufruf ermittelt die Lnge des Strings
        If Invoke(pInterface, vtb, 0&, 0&, VarPtr(cchActual)) = S_OK Then

            ' ist die Lnge grer 0
            If cchActual > 0 Then

                cchString = cchActual
                wzString = String$(cchActual, vbNullChar)

                ' zweiter Aufruf liet den String aus
                If Invoke(pInterface, vtb, cchString, StrPtr(wzString), _
                    VarPtr(cchActual)) = S_OK Then

                    GetCodecInfoString = Left$(wzString, cchActual - 1)

                End If

            End If

        End If

    End If

End Function

Public Function GetCodecInfoGUID_ARRAY(ByVal pInterface As Long, _
    ByVal vtb As vtb_Interfaces) As GUID()

    Dim tGUID() As GUID
    Dim cchGuids As Long
    Dim cchActual As Long

    ' ist ein Pointer auf ein Interface vorhanden
    If pInterface <> 0& Then

        ' erster Aufruf ermittelt die Gre des Arrays
        If Invoke(pInterface, vtb, 0&, 0&, VarPtr(cchActual)) = S_OK Then

            ' ist die Anzahl der Items im Array grer 0
            If cchActual > 0 Then
            
                cchGuids = cchActual
                ReDim tGUID(cchActual - 1)
            
                ' zweiter Aufruf liet das Array aus
                If Invoke(pInterface, vtb, cchGuids, VarPtr(tGUID(0)), _
                    VarPtr(cchActual)) = S_OK Then

                    GetCodecInfoGUID_ARRAY = tGUID

                End If
            
            End If
            
        End If
        
    End If

End Function

Public Function GetCodecInfoBool(ByVal pInterface As Long, _
    ByVal vtb As vtb_Interfaces) As Boolean

    Dim fSupportBool As Long
    
    ' ist ein Pointer auf ein Interface vorhanden
    If pInterface <> 0& Then
    
        If Invoke(pInterface, vtb, VarPtr(fSupportBool)) = S_OK Then
        
            GetCodecInfoBool = CBool(fSupportBool)
        
        End If
        
    End If
    
End Function

Public Function GetCodecInfoLong(ByVal pInterface As Long, _
    ByVal vtb As vtb_Interfaces) As Long

    Dim lValueLong As Long
    
    ' ist ein Pointer auf ein Interface vorhanden
    If pInterface <> 0& Then
    
        If Invoke(pInterface, vtb, VarPtr(lValueLong)) = S_OK Then
        
            GetCodecInfoLong = lValueLong
        
        End If
        
    End If
    
End Function

Public Function GetCodecInfoCLSID(ByVal pInterface As Long, _
    ByVal vtb As vtb_Interfaces) As CLSID

    Dim lValueCLSID As CLSID
    
    ' ist ein Pointer auf ein Interface vorhanden
    If pInterface <> 0& Then
    
        If Invoke(pInterface, vtb, VarPtr(lValueCLSID)) = S_OK Then
        
            GetCodecInfoCLSID = lValueCLSID
        
        End If
        
    End If

End Function

Public Function GetCodecInfoGUID(ByVal pInterface As Long, _
    ByVal vtb As vtb_Interfaces) As GUID

    Dim lValueGUID As GUID
    
    ' ist ein Pointer auf ein Interface vorhanden
    If pInterface <> 0& Then
    
        If Invoke(pInterface, vtb, VarPtr(lValueGUID)) = S_OK Then
        
            GetCodecInfoGUID = lValueGUID
        
        End If
        
    End If

End Function
